@@ -3,6 +3,7 @@ mod statsi64;
 mod vecf64;
 mod indices;
 mod vecu8;
+mod vecvecu8;
 mod mutvec;
 mod vecvec;
 pub mod functions;
@@ -117,7 +118,10 @@ pub trait Vecf64 {
     fn varea(self, v:&[f64]) -> f64;
     /// Area proportional to the swept arc
     fn varc(self, v:&[f64]) -> f64; 
- 
+    /// Vector similarity in the interval [0,1]: (1+cos(theta))/2
+    fn vsim(self, v:&[f64]) -> f64;
+    /// Vector dissimilarity in the interval [0,1]: (1-cos(theta))/2
+    fn vdisim(self, v:&[f64]) -> f64; 
     /// Correlation
     fn correlation(self, _v: &[f64]) -> f64; 
     /// Kendall's tau-b (rank order) correlation
@@ -136,10 +140,12 @@ pub trait Vecf64 {
 
     /// Binary search for insert index I in sorted vector 
     fn binsearch(self, v: f64) -> usize;
+    /// Merges two ascending sorted vectors
+    fn merge(self, v: &[f64]) -> Vec<f64>;
     /// Sort vector in a standard way
     fn sortf(self) -> Vec<f64>;
     /// Sorted vector, is wrapper for mergesort below
-    fn sortm(self) -> Vec<f64>;
+    fn sortm(self, ascending:bool) -> Vec<f64>;
     /// Ranking with only n*log(n) complexity, using 'mergesort'
     fn mergerank(self) -> Vec<usize>;
     /// Immutable merge sort, makes a sort index
@@ -156,8 +162,12 @@ pub trait Vecu8 {
     fn dotp(self, v: &[f64]) -> f64;
     /// Scalar product of two u8 vectors -> u64
      fn dotpu8(self, v: &[u8]) -> u64;
+    /// Cosine between u8 and f64 vectors
+    fn cosine(self, v: &[f64]) -> f64;
     /// Cosine between two u8 vectors
-    fn cosine(self, v: &[u8]) -> f64;
+    fn cosineu8(self, v: &[u8]) -> f64;
+    /// Vector subtraction
+    fn vsub(self, v: &[f64]) -> Vec<f64>;
     /// Vector subtraction
     fn vsubu8(self, v: &[u8]) -> Vec<f64>;
     /// Vector addition
@@ -171,7 +181,11 @@ pub trait Vecu8 {
     /// Euclidian distance to &[u8]
     fn vdistu8(self, v:&[u8]) -> f64;
     /// Euclidian distance between byte vectors
-    fn vdistsq(self, v: &[u8]) -> u64;    
+    fn vdistsq(self, v: &[u8]) -> u64; 
+    /// Vector similarity S in the interval [0,1]: S = (1+cos(theta))/2
+    fn vsim(self, v:&[f64]) -> f64;
+    /// We define vector dissimilarity D in the interval [0,1]: D = 1-S = (1-cos(theta))/2
+    fn vdisim(self, v:&[f64]) -> f64;   
     /// Area proportional to the swept arc
     fn varc(self, v:&[f64]) -> f64; 
 
@@ -217,8 +231,10 @@ pub trait VecVecu8 {
     fn nxnonmember(self, p:&[f64]) -> Vec<f64>;
     /// Weighted eccentricity vector for a non member point
     fn wnxnonmember(self, ws:&[f64], p:&[f64]) -> Vec<f64>; 
-    /// Weighted geometric median, sorted eccentricities magnitudes, cummulative weights
-    fn wsortedeccs(self, ws: &[f64], eps:f64) -> ( Vec<f64>,Vec<f64>,Vec<f64> ); 
+    /// Weighted geometric median, sorted eccentricities magnitudes, cpdf of the weights
+    fn wsortedeccs(self, ascending:bool, ws: &[f64], eps:f64) -> ( Vec<f64>,Vec<f64>,Vec<f64> ); 
+    /// Sorted cosines magnitudes
+    fn wsortedcos(self, ascending:bool, medmed: &[f64], zeromed: &[f64], ws: &[f64]) -> ( Vec<f64>,Vec<f64> ); 
     /// Geometric median of a set of nD points
     fn gmedian(self, eps:f64) -> Vec<f64>; 
     /// The weighted geometric median
@@ -255,9 +271,11 @@ pub trait VecVec {
     /// especially for large numbers of points.
     fn exacteccs(self, eps: f64) -> Vec<Vec<f64>>;
     /// Returns ( gm, sorted eccentricities magnitudes )
-    fn sortedeccs(self, eps:f64) -> ( Vec<f64>,Vec<f64> );
-    /// ( wgm, sorted eccentricities magnitudes, associated cpdf of weights )
-    fn wsortedeccs(self, ws: &[f64], eps:f64) -> ( Vec<f64>,Vec<f64>,Vec<f64> ); 
+    fn sortedeccs(self, ascending:bool, eps:f64) -> ( Vec<f64>,Vec<f64> );
+    /// ( wgm, sorted eccentricities magnitudes, associated cpdf )
+    fn wsortedeccs(self, ascending:bool, ws: &[f64], eps:f64) -> ( Vec<f64>,Vec<f64>,Vec<f64> ); 
+    /// Sorted cosines magnitudes and cpdf, needs central median
+    fn wsortedcos(self, ascending:bool, medmed: &[f64], med: &[f64], ws: &[f64]) -> ( Vec<f64>,Vec<f64> ); 
     /// Next approx median point from this member point given by its indx
     fn nxmember(self, indx: usize) -> Vec<f64>;
     /// Ecentricity of a member point given by its indx
@@ -292,7 +310,7 @@ pub trait Indices {
     /// Reverse index
     fn revindex(self) -> Vec<usize>;
     /// Collects values from `v` as per indices in self.
-    fn unindex(self, v:&[f64]) -> Vec<f64>;
+    fn unindex(self, ascending:bool, v:&[f64]) -> Vec<f64>;
     /// Pearson's correlation coefficient of two slices, typically the ranks.  
     fn ucorrelation(self, v: &[usize]) -> f64;  
-}
\ No newline at end of file
+}
