@@ -1,4 +1,4 @@
-use crate::{MutVectors, Vecf64, Indices};
+use crate::{MutVectors, Vecf64, Indices, functions::emsg};
 
 impl Vecf64 for &[f64] {
 
@@ -89,7 +89,7 @@ impl Vecf64 for &[f64] {
     }
 
     /// Area of a parallelogram between two vectors.
-    /// Same as the magnitude of their cross product.
+    /// Same as the magnitude of their cross product |a ^ b| = |a||b|sin(theta).
     /// Attains maximum `|a|.|b|` when the vectors are othogonal.
     fn varea(self, v:&[f64]) -> f64 {
         (self.vmagsq()*v.vmagsq() - self.dotp(v).powi(2)).sqrt()
@@ -97,10 +97,19 @@ impl Vecf64 for &[f64] {
 
     /// Area proportional to the swept arc up to angle theta. 
     /// Attains maximum of `2|a||b|` when the vectors have opposite orientations.
-    /// This is really |a||b|(1-cos(theta))
+    /// This is really |a||b|(1-cos(theta)) = 2|a||b|D
     fn varc(self, v:&[f64]) -> f64 { 
-        (self.vmagsq()*v.vmagsq()).sqrt() - self.dotp(v)
+         (self.vmagsq()*v.vmagsq()).sqrt() - self.dotp(v)
     }
+
+    /// We define vector similarity S in the interval [0,1] as
+    /// S = (1+cos(theta))/2
+    fn vsim(self, v:&[f64]) -> f64 { (1.0+self.cosine(&v))/2.0 }
+
+    /// We define vector dissimilarity D in the interval [0,1] as
+    /// D = 1-S = (1-cos(theta))/2
+    fn vdisim(self, v:&[f64]) -> f64 { (1.0-self.cosine(&v))/2.0 }
+
     
     /// Pearson's correlation coefficient of a sample of two f64 variables.
     /// # Example
@@ -255,25 +264,57 @@ impl Vecf64 for &[f64] {
         self.iter().map(|&x|(x-min)/range).collect()        
     }
 
-    /// Counts how many items in sorted self are less than or equal to 
-    /// the value v, using binary search. 
+    /// Returns index to the first item that is strictly greater than v, 
+    /// using binary search of an ascending sorted list.
+    /// When none are greater, returns self.len(). 
+    /// User must check for this index overflow: if the returned index == 0, then v was below the list,
+    /// else use index-1 as a valid index to the last item that is less than or equal to v.
+    /// This then is the right index to use for looking up cummulative probability density functions. 
     fn binsearch(self, v: f64) -> usize {
-        let mut lo = 0_usize; // index of the first item
-        let mut hi = self.len()-1; // index of the last item
-        if v < self[0] { return 0_usize }; // v is less than the first
-        if v >= self[hi] { return hi+1 }; // v is at the top or over
+        let n = self.len();
+        if n < 2 { emsg(file!(), line!(), "binsearch list is too short!"); }
+        if v < self[0] { return 0_usize }; // v is smaller than the first item
+        let mut hi = n-1; // valid index of the last item
+        if v > self[hi] { return n }; // indicates that v is greater than the last item
+        let mut lo = 0_usize; // initial index of the low limit 
+    
         loop {
             let gap = hi - lo;
-            if gap == 1 { return hi }
-            let tryi = lo+gap/2;          
-            // if value is above or equal, raise the low limit.
-            // counts also repeating equal values. 
-            if v >= self[tryi] { lo = tryi; continue };                 
-            // else value is strictly below, reduce the high limit
-            hi = tryi    
+            if gap <= 1 { return hi }
+            let tryi = lo+gap/2; 
+            // if tryi index's value is above v, reduce the high index
+            if self[tryi] > v { hi = tryi; continue }            
+            // else indexed value is not greater than v, raise the low index;
+            // jumps also repeating equal values. 
+            lo = tryi
         }  
     }
 
+    /// Merges two ascending sorted vectors &[f64]
+    fn merge(self, v: &[f64]) -> Vec<f64> {
+        let mut resvec:Vec<f64> = Vec::new();
+        let l1 = self.len();
+        let l2 = v.len();
+        let mut i1 = 0;
+        let mut i2 = 0;
+        loop {
+            if i1 == l1 { // self is now processed
+                for i in i2..l2 { resvec.push(v[i]) } // copy out the rest of v
+                break // and terminate
+            }
+            if i2 == l2 { // v is now processed
+                for i in i1..l1 { resvec.push(self[i])} // copy out the rest of self
+                break // and terminate
+            }
+            if self[i1] < v[i2] { resvec.push(self[i1]); i1 += 1; continue };
+            if self[i1] > v[i2] { resvec.push(v[i2]); i2 += 1; continue }; 
+            // here they are equal, so consume both
+            resvec.push(self[i1]); i1 += 1;
+            resvec.push(v[i2]); i2 += 1
+        }
+        resvec
+    }
+
     /// New sorted vector. Immutable sort.
     /// Copies self and then sorts it in place, leaving self unchanged.
     /// Calls mutsortf and that calls the standard self.sort_unstable_by
@@ -284,8 +325,9 @@ impl Vecf64 for &[f64] {
     }
     /// Immutable sort. Returns new sorted vector, just like 'sortf' above
     /// but using our indexing 'mergesort' below.
-    fn sortm(self) -> Vec<f64> {
-        self.mergesort(0,self.len()).unindex(&self)
+    /// Simply passes the boolean flag 'ascending' onto 'unindex'.
+    fn sortm(self, ascending:bool) -> Vec<f64> {
+        self.mergesort(0,self.len()).unindex(ascending,&self)
     }
 
     /// Ranking of self by inverting the (merge) sort index.  
@@ -299,7 +341,7 @@ impl Vecf64 for &[f64] {
 
     /// Recursive non-destructive merge sort. The data is read-only, it is not moved or mutated. 
     /// Returns vector of indices to self from i to i+n, such that the indexed values are in sort order.  
-    /// Thus we are moving the index values instead of the actual values. 
+    /// Thus we are moving only the index (key) values instead of the actual values. 
     fn mergesort(self, i:usize, n:usize) -> Vec<usize> {
 
         if n == 1 { let res = vec![i]; return res };  // recursion termination
@@ -311,14 +353,14 @@ impl Vecf64 for &[f64] {
         let sv1 = self.mergesort(i, n1); // recursively sort the first half
         let sv2 = self.mergesort(i+n1, n2); // recursively sort the second half 
 
-        // Now we merge the two sorted indexes into one
+        // Now we will merge the two sorted indices into one
         let mut merged:Vec<usize> = Vec::with_capacity(n); 
         let mut k = 0_usize;   // subscript to the first sorted half
         let mut l = 0_usize;   // subscript to the second sorted half
         let mut firsthead;
         let mut secondhead;
         loop {
-            // accessing the data in self only indirectly trough the sort indexes
+            // accessing the data in self only indirectly trough the sort indices
             firsthead = self[sv1[k]];
             secondhead = self[sv2[l]];
              
@@ -326,19 +368,20 @@ impl Vecf64 for &[f64] {
                 merged.push(sv1[k]); k += 1 }
             else if firsthead > secondhead { // pop the second
                 merged.push(sv2[l]); l += 1 } 
-            else { // they are equal, so pop both, keeping their order
-                merged.push(sv1[k]); k += 1;
-                merged.push(sv2[l]); l += 1 }
-            if k == sv1.len() {   // first one is empty, just copy the rest of the second 
+                else { // they are equal, so pop both, keeping their order
+                    merged.push(sv1[k]); k += 1;
+                    merged.push(sv2[l]); l += 1 }
+            if k == sv1.len() {   // first one is now empty, just copy the rest of the second 
                 while l < sv2.len() {  merged.push(sv2[l]); l += 1  };
-                break
+                break // either done pushing or both are empty
             };
-            if l == sv2.len() {   // second one is empty, just copy the rest of the first
+            // second one is empty, first one is non-empty, just copy the rest of the first
+            if l == sv2.len() {   
                 while k < sv1.len() {  merged.push(sv1[k]); k += 1 };
                 break
-            };            
+            }
+            // here both are still non-empty, so go round the merge loop again                      
         }
         return merged
     }
-
 }
\ No newline at end of file
