@@ -1,27 +1,41 @@
-use crate::{Vecu8,Vecf64,functions};
-use functions::emsg;
+use crate::{Vecu8,Vecf64,functions::emsg};
 
 impl Vecu8 for &[u8] {
     /// Scalar multiplication of a vector, creates new vec
     fn smult(self, s:f64) -> Vec<f64> {
-        self.iter().map(|&x| s*x as f64).collect()
+        self.iter().map(|&x| s*(x as f64)).collect()
      }
      /// Scalar addition to a vector, creates new vec
      fn sadd(self, s:f64) -> Vec<f64> {
-        self.iter().map(|&x| s+x as f64).collect()
+        self.iter().map(|&x| s+(x as f64)).collect()
      }
     /// Scalar product.   
     /// Must be of the same length - no error checking (for speed)
     fn dotp(self, v: &[f64]) -> f64 {
-        self.iter().zip(v).map(|(&xi, &vi)| xi as f64 * vi).sum::<f64>()
+        self.iter().zip(v).map(|(&xi, &vi)| (xi as f64)*vi).sum::<f64>()
     }
     /// Scalar product of two (positive) u8 slices.   
     /// Must be of the same length - no error checking (for speed)
     fn dotpu8(self, v: &[u8]) -> u64 {
-        self.iter().zip(v).map(|(&xi, &vi)| (xi * vi)as u64).sum::<u64>()
+        self.iter().zip(v).map(|(&xi, &vi)| (xi as u64)*(vi as u64)).sum::<u64>()
     }
+    /// Cosine between &[u8] and &[f64].
+    fn cosine(self, v: &[f64]) -> f64 {
+        let (mut sxy, mut sy2) = (0_f64, 0_f64);
+        let sx2: f64 = self
+            .iter()
+            .zip(v)
+            .map(|(&ux, &y)| {
+                let x  = ux as f64;             
+                sxy += x * y;
+                sy2 += y * y;
+                x*x
+            })
+            .sum();
+        sxy / (sx2*sy2).sqrt()
+       }
     /// Cosine between two (positive) u8 slices.
-    fn cosine(self, v: &[u8]) -> f64 {
+    fn cosineu8(self, v: &[u8]) -> f64 {
      let (mut sxy, mut sy2) = (0_f64, 0_f64);
      let sx2: f64 = self
          .iter()
@@ -31,18 +45,22 @@ impl Vecu8 for &[u8] {
              let y = uy as f64;
              sxy += x * y;
              sy2 += y * y;
-             x*x as f64
+             x*x
          })
          .sum();
      sxy / (sx2*sy2).sqrt()
     }
+    /// Vector subtraction 
+    fn vsub(self, v: &[f64]) -> Vec<f64> {
+        self.iter().zip(v).map(|(&xi, &vi)| (xi as f64) - vi).collect()
+    }
     /// Vector subtraction (converts results to f64 as they can be negative)
     fn vsubu8(self, v: &[u8]) -> Vec<f64> {
-        self.iter().zip(v).map(|(&xi, &vi)| xi as f64 - vi as f64).collect()
+        self.iter().zip(v).map(|(&xi, &vi)| (xi as f64) - (vi as f64)).collect()
     }
     /// Vector addition ( converts results to f64, as they can exceed 255 )
     fn vadd(self, v: &[u8]) -> Vec<f64> {
-        self.iter().zip(v).map(|(&xi, &vi)| (xi + vi) as f64).collect()
+        self.iter().zip(v).map(|(&xi, &vi)| (xi as f64) + (vi as f64)).collect()
     }
     /// Vector magnitude
     fn vmag(self) -> f64{
@@ -65,7 +83,7 @@ impl Vecu8 for &[u8] {
     fn vdistu8(self, v: &[u8]) -> f64 {
         self.iter()
             .zip(v)
-            .map(|(&xi, &vi)| (xi as f64 - vi as f64).powi(2))
+            .map(|(&xi, &vi)| ((xi as f64)-(vi as f64)).powi(2))
             .sum::<f64>()
             .sqrt()
     }
@@ -79,10 +97,18 @@ impl Vecu8 for &[u8] {
                 (x - y).pow(2) as u64})           
             .sum::<u64>()
     }
-    /// Area proportional to the swept arc/// Area of swept arc between self &[u8] and v:&[f64]
+    /// Area of swept arc between self &[u8] and v:&[f64]
+    /// = |a||b|(1-cos(theta)) = 2|a||b|D
     fn varc(self, v:&[f64]) -> f64 { 
         (self.vmagsq()*v.vmagsq()).sqrt() - self.dotp(v)
-    }
+    } 
+    /// We define vector similarity S in the interval [0,1] as
+    /// S = (1+cos(theta))/2
+    fn vsim(self, v:&[f64]) -> f64 { (1.0+self.cosine(v))/2.0 }
+
+    /// We define vector dissimilarity D in the interval [0,1] as
+    /// D = 1-S = (1-cos(theta))/2
+    fn vdisim(self, v:&[f64]) -> f64 { (1.0-self.cosine(v))/2.0 }
 
     /// Probability density function of bytes data
     fn pdf(self) -> Vec<f64> {  
@@ -142,4 +168,4 @@ impl Vecu8 for &[u8] {
     fn vecu8asvecf64 (self) -> Vec<f64> {
         self.iter().map(| &x | x as f64).collect()
     }
-}
\ No newline at end of file
+}
