@@ -163,34 +163,63 @@ impl VecVec for &[Vec<f64>] {
  
     /// GM and sorted eccentricities magnitudes.
     /// Describing a set of points `self` in n dimensions
-     fn sortedeccs(self, eps:f64) -> ( Vec<f64>,Vec<f64> ) { 
+     fn sortedeccs(self, ascending:bool, eps:f64) -> ( Vec<f64>,Vec<f64> ) { 
         let mut eccs = Vec::with_capacity(self.len());       
         let gm = self.gmedian(eps);
-        for v in self { // collect ecentricities magnitudes
+        for v in self { // collect raw ecentricities magnitudes
             eccs.push(gm.vdist(&v)) 
         }
-        ( gm, eccs.sortm() )
+        ( gm, eccs.sortm(ascending) )
     }
     
     /// Weighted geometric median, sorted eccentricities magnitudes,
-    /// associated cummulative probability density function of the weights
-    fn wsortedeccs(self, ws: &[f64], eps:f64) -> ( Vec<f64>,Vec<f64>,Vec<f64> ) { 
+    /// associated cummulative probability density function in [0,1] of the weights.
+    fn wsortedeccs(self, ascending:bool, ws: &[f64], eps:f64) -> ( Vec<f64>,Vec<f64>,Vec<f64> ) { 
         let mut eccs = Vec::with_capacity(self.len()); 
         let gm = self.wgmedian(ws,eps);
-        for v in self { // collect ecentricities magnitudes
+        for v in self { // collect true ecentricities magnitudes
             eccs.push(gm.vdist(&v)) 
         }
+        // Apply linear transform
+        // eccs = eccs.lintrans();
         // create sort index of the eccs
         let index = eccs.mergesort(0,self.len());
         // pick the associated points weights in the same order as the sorted eccs
-        let mut weights = index.unindex(&ws);
+        let mut weights = index.unindex(ascending,&ws);
         let mut sumw = 0_f64;
         // accummulate the weights 
         for i in 0..weights.len() {
             sumw += weights[i]; 
             weights[i] = sumw
         }
-        ( gm, index.unindex(&eccs), weights )
+        // divide by the sum to get cum. probabilities in [0,1]
+        for i in 0..weights.len() { weights[i] /= sumw };
+        ( gm, index.unindex(ascending,&eccs), weights )
+    }
+
+    /// Sorted cosines magnitudes,
+    /// associated cummulative probability density function in [0,1] of the weights.
+    /// Needs central median
+    fn wsortedcos(self, ascending:bool, medmed: &[f64], zeromed: &[f64], ws: &[f64]) -> ( Vec<f64>,Vec<f64> ) { 
+        let mut coses = Vec::with_capacity(self.len());  
+        for p in self { // collect coses      
+            coses.push(p.vsub(&medmed).vsim(&zeromed)); 
+        }
+        // Apply linear transform
+        // coses = coses.lintrans();
+        // create sort index of the coses
+        let index = coses.mergesort(0,self.len());
+        // pick the associated points weights in the same order as the sorted coses
+        let mut weights = index.unindex(ascending,&ws);
+        let mut sumw = 0_f64;
+        // accummulate the weights to from cpdf
+        for i in 0..weights.len() {
+            sumw += weights[i]; 
+            weights[i] = sumw
+        }
+        // divide by the sum to get cum. probabilities in [0,1]
+        for i in 0..weights.len() { weights[i] /= sumw };
+        ( index.unindex(ascending,&coses), weights )
     }
 
     /// Vector `eccentricity` or measure of  
