@@ -3,8 +3,8 @@
 #[cfg(test)]
 
 use anyhow::{Result};
-use rstats::{Stats,MutVectors,Vecf64,VecVec,Vecu8,Indices};
-use rstats::functions::{GI,GV,genvec};
+use rstats::{Stats,MutVectors,Vecf64,VecVec,Vecu8,VecVecu8,Indices};
+use rstats::functions::{GI,GV,genvec,genvecu8};
 use devtimer::DevTime;
 
 pub const EPS:f64 = 1e-7;
@@ -60,7 +60,7 @@ fn vecf64() -> Result<()> {
    println!("Sort index:{}",GV(v2.mergesort(0,v2.len()))); 
    println!("R reversed:{}",GV(v2.mergerank().revindex()));    
    println!("Mergerank: {}",GV(v2.mergerank()));
-   println!("Mrg.sorted:{}",GV(v2.sortm()));
+   println!("Mrg.reverse sorted:{}",GV(v2.sortm(false)));
    println!("Sorted:    {}",GV(v2.sortf()));
    println!("Pearson's Correlation:\t{}",GI(v1.correlation(&v2))); 
    println!("Kendall's Correlation:\t{}",GI(v1.kendalcorr(&v2)));  
@@ -70,19 +70,22 @@ fn vecf64() -> Result<()> {
         GI(v1.ranks().unwrap().cosine(&v2.ranks().unwrap())));        
    println!("Euclidian distance:\t{}",GI(v1.vdist(&v2)));
    println!("Difference magnitude:\t{}",GI(v1.vsub(&v2).as_slice().vmag()));   
-   println!("Vector difference: {}",GV(v1.vsub(&v2))); 
-   println!("Vector addition:   {}",GV(v1.vadd(&v2)));  
+   println!("Vector difference:\n{}",GV(v1.vsub(&v2))); 
+   println!("Vector addition:\n{}",GV(v1.vadd(&v2)));  
    println!("Scalar product:\t\t{}",GI(v1.dotp(&v2)));
    println!("Parallelogram area:\t{}",GI(v1.varea(&v2))); 
-   println!("Arc area:\t\t{}\n",GI(v1.varc(&v2)));
+   println!("Similarity:\t\t{}",GI(v1.vsim(&v2)));
+   println!("Dissimilarity:\t\t{}\n",GI(v1.vdisim(&v2))); 
+   // println!("Arc area:\t\t{}\n",GI(v1.varc(&v2)));
    Ok(())
 }
 #[test]
 fn vecvec() -> Result<()> { 
    let d = 10_usize;
-   let n = 100_usize;
+   let n = 101_usize;
    println!("testing on a random set of {} points in {} dimensional space",GI(n),GI(d));
    let pt = genvec(d,n,5,17); // random test data 
+   let ptu8 = genvecu8(d,n,5,17); // random u8 dataset 
    let (med,medi,outd,outi) = pt.medoid();
    let (mede,medei,oute,outei) = pt.emedoid(EPS);
    let hcentroid = pt.hcentroid();
@@ -115,13 +118,17 @@ fn vecvec() -> Result<()> {
    let (mu,eccmed) = pt.moe(EPS);
    println!("Eccentricities\t{}",mu);  
    println!("Eccentricities\t{}",eccmed);
-   let (_, seccs) = pt.sortedeccs(EPS); 
-   // println!("Sorted eccs: {}\n", GV(seccs));
+   let (_, seccs) = pt.sortedeccs(true,EPS); 
+   println!("Sorted eccs: {:?}\n", seccs);
    let medcnt = seccs.binsearch(eccmed.median);
    println!("Items smaller or equal to median of eccs: {} last value: {}", GI(medcnt), GI(seccs[medcnt-1]));
    let mut weights = Vec::new();
-   for i in 0..n { weights.push(i as f64) }; // weight data 
-   println!("Weighted median: {}",GV(pt.wgmedian(&weights,EPS)));
+   for i in 1..n+1 { weights.push(i as f64) }; // create test weights data
+   // create pretend median of medians
+   let medmed = vec![0.5_f64;n];
+   let (se, cpdf) = 
+    ptu8.wsortedcos(false, &medmed, &pt.wgmedian(&weights,EPS), &weights);
+   println!("Reverse sorted coses:\n{}\ncpdf:\n{}\n",GV(se),GV(cpdf));
    Ok(())
 }
 
